//+------------------------------------------------------------------+
//|                                                       AMA_DF.mq4 |
//|                                      Copyright  2006, Alexandre |
//|                                       mailto: a_g_j_@hotmail.com |
//+------------------------------------------------------------------+
//---- general properties
#property copyright "Copyright  2006, Alexandre"
#property link      "mailto: a_g_j_@hotmail.com"
//---- indicator properties
#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 ForestGreen  // 'line'  (main)
#property indicator_color2 DeepSkyBlue
#property indicator_color3 DeepPink
#property indicator_color4 PaleGreen 
//---- defines
//---- input parameters
extern bool   LastBarOnly = true;
extern int    Range       = 10; 
extern int    Shift       = 0;     //
extern double ER_Low      = 0.05;
extern double ER_Upp      = 0.95; 
extern double P_G         = 2.0; 
extern double K_F         = 1.0; 
//---- common buffers
//---- indicator buffers
double AMA_Buffer[]; 
double UpSig_Buffer[]; 
double DwSig_Buffer[]; 
double NlSig_Buffer[];
//---- variables
double A = 0.45; // quotation coefficient - value close to optimal 
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
 {
 //---- indicators
  SetIndexStyle(0, DRAW_LINE);
  SetIndexLabel(0, "AMA_MOD"); 
  SetIndexBuffer(0, AMA_Buffer); // main line
  SetIndexDrawBegin(0, Range); 
  SetIndexEmptyValue(0, 0.0); 
  // 
  SetIndexStyle(1, DRAW_ARROW);
  SetIndexLabel(1, "Signal Up"); 
  SetIndexShift(1, Shift);
  //SetIndexArrow(1, 108);             // small arrow up
  SetIndexArrow(1, 159);             // small arrow up
  SetIndexBuffer(1, UpSig_Buffer); 
  SetIndexDrawBegin(1, Range); 
  SetIndexEmptyValue(1, 0.0); 
  // 
  SetIndexStyle(2, DRAW_ARROW);
  SetIndexLabel(2, "Signal Down");
  SetIndexShift(2, Shift);
  //SetIndexArrow(2, 108);             // small arrow down
  SetIndexArrow(2, 159);             // small arrow up
  SetIndexBuffer(2, DwSig_Buffer); 
  SetIndexDrawBegin(2, Range); 
  SetIndexEmptyValue(2, 0.0); 
  // 
  SetIndexStyle(3, DRAW_ARROW);
  SetIndexLabel(3, "Signal Stop");
  SetIndexShift(3, Shift);
  //SetIndexArrow(3, 108);             // small stop-symbol
  SetIndexArrow(3, 159);             // small arrow up
  SetIndexBuffer(3, NlSig_Buffer); 
  SetIndexDrawBegin(3, Range); 
  SetIndexEmptyValue(3, 0.0); 
 //----
  return(0);
 }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
 {
 //----
 //
  //Comment(""); 
 //----
  return(0);
 }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
 {
  int counted_bars = IndicatorCounted();
  int Limit;
  int i, k; 
  double m_pr, d_f; 
  static double v_prev, v_curr, m_pr_old; 
  static bool   run_once; 
 //----
  if (counted_bars < 0) { return(-1); }
  Limit = Bars - counted_bars;
  //
  if (run_once == false) // run once on start
   { 
    k = Limit - Range; 
    m_pr = (High[Limit-Range] + Low[Limit-Range]) / 2.0; 
    v_prev = m_pr; 
   }
  else // run elsewhere
   {
    if (LastBarOnly == false) 
     { k = Limit; v_prev = AMA_Buffer[Limit+1]; }
    else
     { k = 0; v_prev = AMA_Buffer[1]; }
   }
  // 
  m_pr = (High[0] + Low[0]) / 2.0; 
  if (m_pr == m_pr_old) { return(0); } 
  else                  { m_pr_old = m_pr; } 
  // 
  for (i=k; i>=0; i--)
   { 
    v_curr = GetAMAOneStep(v_prev, i, Range, ER_Low, ER_Upp, P_G, A); 
    d_f = (v_curr - v_prev) / Point; 
    //Comment("d_f = ", d_f);
    AMA_Buffer[i] = v_curr; // line 
    if (run_once == false) { v_prev = v_curr; } 
    //
    if ((MathAbs(d_f) > K_F) && (d_f > 0)) 
     { 
      UpSig_Buffer[i] = AMA_Buffer[i]; 
      DwSig_Buffer[i] = -0.1; 
      NlSig_Buffer[i] = -0.1; 
     }
    else if ((MathAbs(d_f) > K_F) && (d_f < 0))
     { 
      UpSig_Buffer[i] = -0.1; 
      DwSig_Buffer[i] = AMA_Buffer[i]; 
      NlSig_Buffer[i] = -0.1; 
     }
    else
     { 
      UpSig_Buffer[i] = -0.1; 
      DwSig_Buffer[i] = -0.1; 
      NlSig_Buffer[i] = AMA_Buffer[i]; 
     }
   }
  //
  if (run_once == false) { run_once = true; } 
 //----
  return(0);
 }
//+------------------------------------------------------------------+
//| Median Prices Adaptive Smoothing - AMA One Step                  |
//+------------------------------------------------------------------+
double GetAMAOneStep(double v_prev, int n_bar, int range, double er_low, 
                                    double er_upp, double pow, double a)
 {
  double v_curr, noise, er, ssc, m_pr, m_pr1; 
 //----
 // calculate efficiency ratio & ssc 
  if (range > 1)
   {
    noise = 0.000000001;
    for (int i=n_bar+range-1; i>=n_bar; i--)
     { 
      m_pr   = (High[i] + Low[i]) / 2.0; 
      m_pr1  = (High[i+1] + Low[i+1]) / 2.0; 
      noise += MathAbs(m_pr - m_pr1); 
     }
    m_pr  = (High[n_bar] + Low[n_bar]) / 2.0; 
    m_pr1 = (High[n_bar+range] + Low[n_bar+range]) / 2.0; 
    er  = MathAbs(m_pr - m_pr1) / noise; 
   }
  else 
   { er = 1.0; } 
  //er  = MathPow(er, pow);  
  ssc = er * er_upp + er_low;
  ssc  = MathPow(ssc, pow); 
  //
  // calculate AMA one step, using explicit/implicit Euler Schema
  m_pr  = (High[n_bar] + Low[n_bar]) / 2.0; 
  m_pr1 = (High[n_bar+1] + Low[n_bar+1]) / 2.0; 
  v_curr = (v_prev + ssc * (a * (m_pr1 - v_prev) + 
           (1.0 - a) * m_pr)) / (1.0 + (1.0 - a) * ssc); 
 //----
  return(v_curr);
 }
//+------------------------------------------------------------------+