//+------------------------------------------------------------------+
//|                                                AMA_DF_Expert.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
//
extern double Lots           = 0.1;
extern bool   LastBarOnly = true; 
extern int    Range       = 10; 
extern int    Shift       = 0;     //
extern double ER_Low      = 0.05;
extern double ER_Upp      = 0.95;
extern double P_G         = 2.0; 
extern double K_F         = 1.0; 
// 
static datetime prev_time; 
static int MAGICMA; 
//
double ama_up, ama_dw, ama_nl; 
//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
 {
  int buys = 0, sells = 0;
 //----
  for (int i = OrdersTotal() - 1; i >= 0; i--)
   {
    if (OrderSelect(i, SELECT_BY_POS,MODE_TRADES) == true) 
     {
      if ((OrderSymbol() == Symbol()) && (OrderMagicNumber() == MAGICMA))
       {
        if (OrderType() == OP_BUY)  { buys++;  } 
        if (OrderType() == OP_SELL) { sells++; }
       }
     }
   }
 //---- return orders volume
  if (buys > 0) { return(buys);   } 
  else          { return(-sells); }
 }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
 { 
  int res;
 //---- sell conditions
  if ((NormalizeDouble(ama_up, 1) == -0.1) && (NormalizeDouble(ama_dw, 1) > 0) && 
                                             (NormalizeDouble(ama_nl, 1) == -0.1)) 
   { res = OrderSend(Symbol(), OP_SELL, Lots, Bid, 3, 0, 0, "", MAGICMA, 0, Red); }
 //---- buy conditions
  if ((NormalizeDouble(ama_up, 1) > 0) && (NormalizeDouble(ama_dw, 1) == -0.1) && 
                                          (NormalizeDouble(ama_nl, 1) == -0.1)) 
   { res = OrderSend(Symbol(), OP_BUY, Lots, Ask, 3, 0, 0, "", MAGICMA, 0, Blue); }
 //----
  return; 
 }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
 { 
 //---- 
  for (int i = OrdersTotal() - 1; i >= 0; i--) 
   {
    if ((OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) == true) 
     {
      if ((OrderMagicNumber() == MAGICMA) && (OrderSymbol() == Symbol())) 
       {
        //---- check order type 
        if (OrderType() == OP_BUY) 
         { 
          if ((NormalizeDouble(ama_up, 1) == -0.1) && (NormalizeDouble(ama_dw, 1) > 0) && 
                                                     (NormalizeDouble(ama_nl, 1) == -0.1)) 
           { OrderClose(OrderTicket(), OrderLots(), Bid, 3, Aqua); } 
         } 
        //----
        if (OrderType() == OP_SELL)
         { 
          if ((NormalizeDouble(ama_up, 1) > 0) && (NormalizeDouble(ama_dw, 1) == -0.1) && 
                                                  (NormalizeDouble(ama_nl, 1) == -0.1)) 
           { OrderClose(OrderTicket(), OrderLots(), Ask, 3, Magenta); } 
         } 
       }
     }
   }
 //---- 
  return; 
 }
//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
 {
  static int sh_cnt = 0; 
  //
  string file_name;
 //---- check for history and trading
  if ((Bars < 100) || (IsTradeAllowed() == false)) { return; } 
 //---- go trading only for first tiks of new bar
  if (prev_time == Time[0]) { return; } 
 //---- get AMA values 
  ama_up = iCustom(NULL, 0, "AMA_DF", LastBarOnly, Range, Shift, ER_Low, ER_Upp, P_G, K_F, 1, 0); 
  ama_dw = iCustom(NULL, 0, "AMA_DF", LastBarOnly, Range, Shift, ER_Low, ER_Upp, P_G, K_F, 2, 0); 
  ama_nl = iCustom(NULL, 0, "AMA_DF", LastBarOnly, Range, Shift, ER_Low, ER_Upp, P_G, K_F, 3, 0); 
 //---- 
  if (IsVisualMode() == true)
   {
    Comment("Open: ama_up = ", ama_up, "; ama_dw = ", ama_dw, "; ama_nl = ", ama_nl);  
    //----
    if ((NormalizeDouble(ama_up, 1) == 0) || (NormalizeDouble(ama_dw, 1) == 0) || 
                                             (NormalizeDouble(ama_nl, 1) == 0)) 
     { 
      sh_cnt++; 
      file_name = "error_" + DoubleToStr(sh_cnt, 0) + ".gif"; 
      WindowScreenShot(file_name, 800, 600); 
     } 
   }
 //---- calculate open orders by current symbol
  if (CalculateCurrentOrders(Symbol()) == 0) { CheckForOpen();  } 
  else                                       { CheckForClose(); } 
 //----
  prev_time = Time[0]; 
 //----
  return; 
 }
//
//+------------------------------------------------------------------+
//| Init function                                                    |
//+------------------------------------------------------------------+
void init() 
 { 
  MathSrand(TimeLocal()); 
 //---- 
  prev_time = Time[0]; 
 //----
  MAGICMA = AccountNumber() + MathRand(); 
  Print("MAGICMA = ", MAGICMA); 
 //----
  return; 
 } 
//+------------------------------------------------------------------+